@@ -1,3 +1,3 @@
-from openqlab.analysis import cavity, phase
+from openqlab.analysis import cavity, phase, squeezing
 from openqlab.analysis.gaussian_beam import GaussianBeam, fit_beam_data
 from openqlab.analysis.servo_design import ServoDesign
