@@ -4,9 +4,12 @@ import logging as log
 from typing import List, Optional, Tuple
 from warnings import warn
 
+import matplotlib
 import matplotlib.pyplot as plt
 import numpy as np
-from pandas import Series
+from numpy import float64, ndarray
+from pandas.core.indexes.numeric import Float64Index
+from pandas.core.series import Series
 from scipy.optimize import least_squares
 from scipy.signal import find_peaks, peak_widths, savgol_filter
 
@@ -16,13 +19,14 @@ from openqlab.io.data_container import DataContainer
 #
 # Modematching methods
 #
-def modematching(
+def modematching(  # pylint: disable=too-many-branches
     data: Series,
     plot: bool = False,
     U_max: Optional[float] = None,
     offset: Optional[float] = None,
     rel_prominence: float = 0.02,
     without_main_peaks: bool = False,
+    ax: matplotlib.axes.Axes = None,
 ) -> float:
     """Calculate the mode matching.
 
@@ -53,6 +57,8 @@ def modematching(
         of small peaks.
     without_main_peaks : bool
         Deprecated! Is assumed if U_max is set.
+    ax (Axes, optional):
+        Provide an Axes for advanced figures.
 
     Returns
     -------
@@ -68,6 +74,8 @@ def modematching(
         )
     if U_max is not None:
         without_main_peaks = True
+    if ax is None:
+        _, ax = plt.subplots()
 
     data = data.dropna()
 
@@ -117,14 +125,15 @@ def modematching(
 
     # Plotting
     if plot:
-        _main_plot(data, peaks=peaks, main_peaks=peaks[main_mode])
-
-        if not without_main_peaks:
+        if without_main_peaks:
+            _main_plot(data, peaks=peaks, ax=ax)
+        else:
+            _main_plot(data, peaks=peaks, main_peaks=peaks[main_mode], ax=ax)
             index_first, index_last = peaks[main_mode]
-            plt.axvline(x=data.index[index_first], color="gray")
-            plt.axvline(x=data.index[index_last], color="gray")
-            plt.axvspan(data.index[0], data.index[index_first], color="gray", alpha=0.5)
-            plt.axvspan(data.index[index_last], data.index[-1], color="gray", alpha=0.5)
+            ax.axvline(x=data.index[index_first], color="gray")
+            ax.axvline(x=data.index[index_last], color="gray")
+            ax.axvspan(data.index[0], data.index[index_first], color="gray", alpha=0.5)
+            ax.axvspan(data.index[index_last], data.index[-1], color="gray", alpha=0.5)
 
     print(f"Mode matching: {round(mode_matching*100, 2)}%")
 
@@ -133,15 +142,16 @@ def modematching(
 
 def _main_plot(
     data: Series,
+    ax: matplotlib.axes.Axes,
     peaks: Optional[np.ndarray] = None,
     main_peaks: Optional[np.ndarray] = None,
 ):
-    axes = data.plot()
-    axes.set_xlim(data.index[0], data.index[-1])
+    data.plot(ax=ax)
+    ax.set_xlim(data.index[0], data.index[-1])
     if peaks is not None:
-        data.iloc[peaks].plot(style=".")
+        data.iloc[peaks].plot(style=".", ax=ax)
     if main_peaks is not None:
-        data.iloc[main_peaks].plot(style="o")
+        data.iloc[main_peaks].plot(style="o", ax=ax)
 
 
 def _find_peaks(
@@ -161,7 +171,9 @@ def _find_peaks(
 #
 # Finesse methods
 #
-def finesse(data: Series, plot: bool = False) -> List[float]:
+def finesse(
+    data: Series, plot: bool = False, ax: matplotlib.axes.Axes = None
+) -> List[float]:
     """Finesse calculation using a cavity scan.
 
     Parameters
@@ -170,6 +182,8 @@ def finesse(data: Series, plot: bool = False) -> List[float]:
         data is the amplitude column with two main modes.
     plot : bool
         plot is giving out a plot to make shure the algorithm has found the correct points.
+    ax (Axes, optional):
+        Provide an Axes for advanced figures.
 
     Returns
     -------
@@ -178,11 +192,13 @@ def finesse(data: Series, plot: bool = False) -> List[float]:
     """
     if len(data.shape) != 1:
         raise ValueError("The DataFrame should only contain one single column.")
+    if ax is None:
+        _, ax = plt.subplots()
     data = data.dropna()
     _adjust_peak_sign(data)
 
     peaks, main_mode = _find_peaks(data, 0.9)
-    result = _calculate_finesse(data, peaks, main_mode, plot)
+    result = _calculate_finesse(data, peaks, main_mode, ax, plot)
     print(
         f"Finesse first peak: {round(result[0], 2)}, second peak: {round(result[1], 2)}"
     )
@@ -190,7 +206,11 @@ def finesse(data: Series, plot: bool = False) -> List[float]:
 
 
 def _calculate_finesse(
-    data: Series, peaks: np.ndarray, main_mode: np.ndarray, plot: bool = False
+    data: Series,
+    peaks: np.ndarray,
+    main_mode: np.ndarray,
+    ax: matplotlib.axes.Axes,
+    plot: bool = False,
 ) -> List[float]:
     peak_data = peak_widths(data, peaks)
     peak_fwhm = peak_data[0]
@@ -200,14 +220,14 @@ def _calculate_finesse(
     fsr = peaks[main_mode[1]] - peaks[main_mode[0]]
 
     if plot:
-        _main_plot(data, main_peaks=peaks[main_mode])
+        _main_plot(data, main_peaks=peaks[main_mode], ax=ax)
         for x in np.concatenate([peaks_left[main_mode], peaks_right[main_mode]]):
-            plt.axvline(x=data.index[int(x)], ls=":", color="green")
+            ax.axvline(x=data.index[int(x)], ls=":", color="green")
 
     return [fsr / main_width[0], fsr / main_width[1]]
 
 
-def _adjust_peak_sign(data: Series):
+def _adjust_peak_sign(data: Series) -> None:
     minimum = np.min(data)
     maximum = np.max(data)
 
@@ -219,8 +239,12 @@ def _adjust_peak_sign(data: Series):
 # Linewidth methods
 #
 def linewidth(
-    err: Series, aux: Series, mod_freq, plot: bool = False, fwhm_guess: float = 1e6
-):
+    err: Series,
+    aux: Series,
+    mod_freq: float,
+    plot: bool = False,
+    fwhm_guess: float = 1e6,
+) -> float64:
     # _adjust_peak_sign(err)
     _adjust_peak_sign(aux)
 
@@ -314,16 +338,16 @@ def linewidth(
     return fwhm * 1e6
 
 
-def _line(x, p):
+def _line(x: Float64Index, p: ndarray) -> Float64Index:
     """Fit the line in the error signal to find zero."""
     return x + p[0]
 
 
-def _resid_line(p, y, x):
+def _resid_line(p: ndarray, y: Series, x: Float64Index) -> Series:
     return y - _line(x, p)
 
 
-def _fit_line(data):
+def _fit_line(data: Series) -> Float64Index:
     p = [0]
 
     # optimization #
@@ -333,18 +357,24 @@ def _fit_line(data):
     return fit
 
 
-def _lorentzian(x, p):
+def _lorentzian(x: ndarray, p: ndarray) -> ndarray:
     """Define the lorentzian fit."""
     numerator = p[1] ** 2
     denominator = (x - p[0]) ** 2 + p[1] ** 2
     return p[2] * (numerator / denominator) - p[3]
 
 
-def _residuals(p, y, x):
+def _residuals(p: ndarray, y: ndarray, x: ndarray) -> ndarray:
     return y - _lorentzian(x, p)
 
 
-def _fit_lorentzian(data, time_array, df, peak, fwhm_guess):
+def _fit_lorentzian(
+    data: Series,
+    time_array: Float64Index,
+    df: float64,
+    peak: float64,
+    fwhm_guess: float,
+) -> Tuple[ndarray, ndarray, float64, float64]:
     data = np.array(data)
     freq = np.array(time_array) * df
 
