@@ -1,11 +1,14 @@
-from typing import List, Tuple
+from typing import List, Tuple, Union
 
 import numpy as np
 import pandas as pd
+from numpy import ndarray
+from pandas.core.indexes.numeric import Float64Index
 from scipy.optimize import curve_fit
 
 from openqlab.analysis.electromagnetic_wave import ElectromagneticWave
 from openqlab.conversion.utils import human_readable
+from openqlab.io.data_container import DataContainer
 from openqlab.plots.gaussian_beam import beam_profile
 
 
@@ -14,7 +17,7 @@ class GaussianBeam(ElectromagneticWave):
     Represents a Gaussian beam with a complex gaussian beam parameter q.
     """
 
-    def __init__(self, q, wavelength):
+    def __init__(self, q: complex, wavelength: float) -> None:
         if wavelength <= 0:
             raise ValueError("Wavelength must be a positiv number")
         super().__init__(wavelength)
@@ -41,7 +44,9 @@ class GaussianBeam(ElectromagneticWave):
         """
         return GaussianBeam(self._q + d, self._wavelength)
 
-    def get_profile(self, zpoints):
+    def get_profile(
+        self, zpoints: Union[Float64Index, ndarray, float]
+    ) -> Union[ndarray, Float64Index, float]:
         """
         Returns the beam width at points zpoints along the beam axis.
         """
@@ -49,49 +54,48 @@ class GaussianBeam(ElectromagneticWave):
         return self.w0 * np.sqrt(1 + quotient ** 2)
 
     @property
-    def wavelength(self) -> float:
+    def wavelength(self) -> Union[float, np.ndarray]:
         return self._wavelength
 
     @property
-    def w0(self) -> float:
+    def w0(self) -> Union[float, np.ndarray]:
         """
         Waist size in meters.
         """
         return np.sqrt(self._q.imag * self._wavelength / np.pi)
 
     @property
-    def z0(self) -> float:
+    def z0(self) -> Union[float, np.ndarray]:
         """
         The position of the beam waist on the z-axis in meters.
         """
         return -self._q.real  # pylint: disable=invalid-unary-operand-type
 
     @property
-    def zR(self):
+    def zR(self) -> Union[float, np.ndarray]:
         return self._q.imag
 
     @property
-    def R(self):
+    def R(self) -> float:
         qi = 1 / self._q
         if qi.real == 0:
             return np.infty
         return 1 / (qi.real)
 
     @property
-    def w(self):
+    def w(self) -> Union[float, np.ndarray]:
         return self.get_profile(0.0)
 
     @property
-    def divergence(self):
-        """Beam divergence in radians.
-
-        """
+    def divergence(self) -> Union[float, np.ndarray]:
+        """Beam divergence in radians."""
         return np.arctan(self._wavelength / np.pi / self.w0)
 
-    def __repr__(self):
-        return "w0={w0} @ z0={z0}".format(
-            w0=human_readable(self.w0, "m"), z0=human_readable(self.z0, "m")
-        )
+    # TODO enable with useful types
+    # def __repr__(self) -> str:
+    # return "w0={w0} @ z0={z0}".format(
+    # w0=human_readable(self.w0, "m"), z0=human_readable(self.z0, "m")
+    # )
 
 
 def fit_beam_data(
@@ -101,8 +105,8 @@ def fit_beam_data(
     guess_w0: float = 300e-6,
     guess_z0: float = 0.0,
     plot: bool = True,
-    print_results=True,
-) -> pd.DataFrame:
+    print_results: bool = True,
+) -> DataContainer:
     """
 
     Parameters
@@ -143,11 +147,14 @@ def fit_beam_data(
             index=["w0", "w0_error", "z0", "z0_error"],
         )
 
-    results = data.apply(_fit_beam)
+    results = DataContainer(data.apply(_fit_beam))
     results.header["wavelength"] = wavelength
     if plot:
+        label_template = "{name}: {w0}m @ {z0}m"
         beams = {
-            name: GaussianBeam.from_waist(
+            label_template.format(
+                name=name, w0=human_readable(beam["w0"]), z0=human_readable(beam["z0"])
+            ): GaussianBeam.from_waist(
                 w0=beam["w0"], z0=beam["z0"], wavelength=wavelength
             )
             for name, beam in results.iteritems()
