@@ -1,6 +1,8 @@
 """Phase module for calculations regarding the phase."""
+from typing import Union
+
 from numpy import sign
-from pandas import Series
+from pandas import DataFrame, Series
 
 
 def accumulated_phase(data: Series, limit: float = 340) -> None:
@@ -28,7 +30,9 @@ def accumulated_phase(data: Series, limit: float = 340) -> None:
             data.iloc[i:] += 360 * sign(data.iloc[i - 1] - data.iloc[i])
 
 
-def clamp_phase(phase):
+def clamp_phase(
+    phase: Union[float, Series, DataFrame]
+) -> Union[float, Series, DataFrame]:
     """
     Returns phase with all values mapped to between +/- 180 degrees.
     """
