@@ -1,11 +1,16 @@
 """:obj:`openqlab.ServoDesign` helps with designing a standard servo circuit."""
 from abc import ABC, abstractmethod
+from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
 from warnings import warn
 
 import numpy as np
 import pandas as pd
+from matplotlib.figure import Figure
+from numpy import ndarray
+from pandas import DataFrame, Series
+from pandas.core.indexes.numeric import Float64Index
 from scipy import signal
-from tabulate import tabulate
+from tabulate import tabulate  # type: ignore
 
 from openqlab import plots
 from openqlab.conversion import db
@@ -13,7 +18,7 @@ from openqlab.conversion.utils import human_readable
 from openqlab.io import DataContainer
 
 
-def _handle_keysight_files(df):
+def _handle_keysight_files(df: DataContainer) -> DataContainer:
     value = df.copy()
     # Extra handling for Keysight data_files
     columns = ["Amplitude (Vpp)", "Gain (dB)", "Phase (deg)"]
@@ -38,13 +43,18 @@ class Filter(ABC):
         Gain
     """
 
-    def __init__(self, corner_frequency, second_parameter=None, enabled=True):
+    def __init__(
+        self,
+        corner_frequency: float,
+        second_parameter: Optional[float] = None,
+        enabled: bool = True,
+    ) -> None:
         self._corner_frequency = corner_frequency
         self._second_parameter = second_parameter
         self._enabled = enabled
         self.update()
 
-    def update(self):
+    def update(self) -> None:
         z, p, k = self.calculate()
         self._zeros = np.atleast_1d(z)
         self._poles = np.atleast_1d(p)
@@ -80,7 +90,7 @@ class Filter(ABC):
         return self._corner_frequency
 
     @corner_frequency.setter
-    def corner_frequency(self, value):
+    def corner_frequency(self, value: float):
         self._corner_frequency = value
         self.update()
 
@@ -89,40 +99,42 @@ class Filter(ABC):
         return self._second_parameter
 
     @second_parameter.setter
-    def second_parameter(self, value):
+    def second_parameter(self, value: Optional[float]):
         self._second_parameter = value
         self.update()
 
     @property
     @abstractmethod
-    def description(self):
+    def description(self) -> str:
         pass
 
     @property
-    def description_long(self):
+    def description_long(self) -> str:
         # what is this for?
         return self.description
 
     @property
-    def zeros(self):
+    def zeros(self) -> ndarray:
         return self._zeros
 
     @property
-    def poles(self):
+    def poles(self) -> ndarray:
         return self._poles
 
     @property
-    def gain(self):
+    def gain(self) -> float:
         return self._gain
 
-    def discrete_SOS(self, sampling_frequency):  # pylint: disable=invalid-name
+    def discrete_SOS(
+        self, sampling_frequency: float
+    ) -> ndarray:  # pylint: disable=invalid-name
         """
         Return a discrete-time second order section of this filter, with sampling
         frequency `sampling_frequency`.
         """
         return signal.zpk2sos(*self.discrete_zpk(sampling_frequency))
 
-    def discrete_zpk(self, sampling_frequency):
+    def discrete_zpk(self, sampling_frequency: float) -> Tuple[ndarray, ndarray, float]:
         """
         Return the discrete-time transfer function of this filter, evaluated
         for a sampling frequency of `sampling_frequency`.
@@ -130,7 +142,7 @@ class Filter(ABC):
         z, p, k = self._prewarp(sampling_frequency)
         return signal.bilinear_zpk(z, p, k, sampling_frequency)
 
-    def _prewarp(self, sampling_frequency):
+    def _prewarp(self, sampling_frequency: float) -> Tuple[ndarray, ndarray, float]:
         """
         Prewarp frequencies of poles and zeros, to correct for the nonlinear
         mapping of frequencies between continuous-time and discrete-time domain.
@@ -159,10 +171,10 @@ class Filter(ABC):
 
         # since we're calculating in Hz, we need to scale the gain as well
         # by 2pi for each pole and 1/2pi for each zero
-        gain = self._gain * (2 * np.pi) ** self._relative_degree()
+        gain: float = self._gain * (2 * np.pi) ** self._relative_degree()
         return warp(self._zeros), warp(self._poles), gain
 
-    def _relative_degree(self):
+    def _relative_degree(self) -> int:
         return len(self._poles) - len(self._zeros)
 
 
@@ -178,7 +190,7 @@ class Integrator(Filter):
         Frequency were the ~1/f slope starts, defaults to 0.001 * `corner_frequency`.
     """
 
-    def calculate(self):
+    def calculate(self) -> Tuple[float, float, float]:
         z = -self.corner_frequency
         if self.second_parameter is None:
             self.second_parameter = self.corner_frequency * 0.001
@@ -187,7 +199,7 @@ class Integrator(Filter):
         return z, p, k
 
     @property
-    def description(self):
+    def description(self) -> str:
         return "Int {0}, Slope {1}".format(
             human_readable(self.corner_frequency, "Hz"),
             human_readable(self.second_parameter, "Hz"),
@@ -216,7 +228,7 @@ class Differentiator(Filter):
 
     """
 
-    def calculate(self):
+    def calculate(self) -> Tuple[float, float, float]:
         z = -self.corner_frequency
         if self.second_parameter is None:
             self.second_parameter = self.corner_frequency * 10
@@ -225,7 +237,7 @@ class Differentiator(Filter):
         return z, p, k
 
     @property
-    def description(self):
+    def description(self) -> str:
         return "Diff {0}, Slope {1}".format(
             human_readable(self.corner_frequency, "Hz"),
             human_readable(self.second_parameter, "Hz"),
@@ -253,14 +265,19 @@ class Lowpass(Filter):
 
     """
 
-    def __init__(self, corner_frequency, second_parameter=0.707, enabled=True):
+    def __init__(
+        self,
+        corner_frequency: float,
+        second_parameter: float = 0.707,
+        enabled: bool = True,
+    ) -> None:
         super().__init__(corner_frequency, second_parameter, enabled)
 
-    def calculate(self):
-        z = []
+    def calculate(self) -> Tuple[List[Any], List[complex], float]:
+        z: List = []
         corner_frequency = self.corner_frequency
         second_parameter = self.second_parameter
-        p = [
+        p: List[complex] = [
             -corner_frequency / (2 * second_parameter)
             + ((corner_frequency / (2 * second_parameter)) ** 2 - corner_frequency ** 2)
             ** 0.5,
@@ -273,7 +290,7 @@ class Lowpass(Filter):
         return z, p, k
 
     @property
-    def description(self):
+    def description(self) -> str:
         return "LP2 {0}, Q={1:.4g}".format(
             human_readable(self.corner_frequency, "Hz"), self.second_parameter
         )
@@ -301,10 +318,15 @@ class Notch(Filter):
         Quality factor of the notch filter. Defaults to 1. Referenced as `Q` in description.
     """
 
-    def __init__(self, corner_frequency, second_parameter=1, enabled=True):
+    def __init__(
+        self,
+        corner_frequency: float,
+        second_parameter: float = 1,
+        enabled: bool = True,
+    ) -> None:
         super().__init__(corner_frequency, second_parameter, enabled)
 
-    def calculate(self):
+    def calculate(self) -> Tuple[List[complex], List[complex], float]:
         corner_frequency = self.corner_frequency
         second_parameter = self.second_parameter
         z = [corner_frequency * 1j, -corner_frequency * 1j]
@@ -320,7 +342,7 @@ class Notch(Filter):
         return z, p, k
 
     @property
-    def description(self):
+    def description(self) -> str:
         return "Notch {0}, Q={1:.4g}".format(
             human_readable(self.corner_frequency, "Hz"), self.second_parameter
         )
@@ -354,7 +376,7 @@ class ServoDesign:
     MAX_FILTERS = 5
     SAMPLING_RATE = 200e3  # ADwin usually runs with 200 kHz
 
-    def __init__(self):
+    def __init__(self) -> None:
         self._plant = None
         self.clear()
 
@@ -363,7 +385,9 @@ class ServoDesign:
     ####################################
 
     @property
-    def filters(self):
+    def filters(
+        self,
+    ) -> Sequence[Optional[Filter]]:
         """
         Return the global class field filterListself.
 
@@ -374,24 +398,29 @@ class ServoDesign:
         """
         return self._filters
 
-    def clear(self):
-        self._filters = [None] * self.MAX_FILTERS
+    def clear(self) -> None:
+        self._filters: List[Optional[Filter]] = [None] * self.MAX_FILTERS
         self.gain = 1.0
 
-    def _get_first_none_entry(self):
+    def _get_first_none_entry(self) -> Optional[int]:
         for i in range(self.MAX_FILTERS):
             if self.filters[i] is None:
                 return i
         return None
 
-    def _add_filter_on_index(self, filter_, index):  # pylint: disable=redefined-builtin
+    def _add_filter_on_index(
+        self, filter_: Filter, index: int
+    ) -> None:  # pylint: disable=redefined-builtin
         if index >= self.MAX_FILTERS:
             raise IndexError("Max {0} filters are allowed.".format(self.MAX_FILTERS))
         self._filters[index] = filter_
 
     def add(
-        self, filter, index=None, override=False
-    ):  # pylint: disable=redefined-builtin
+        self,
+        filter: Filter,
+        index: Optional[int] = None,
+        override: bool = False,
+    ) -> None:  # pylint: disable=redefined-builtin
         """
         Add a filter to the servo. Up to {0} filters can be added. If the list is full and not index is provided, the filter at the last index may be overriden, depending on whether `override` has been set to true or false.
 
@@ -428,7 +457,7 @@ class ServoDesign:
                 "No filter was added, list was full. You might wanna set `override=True` or remove a filter."
             )
 
-    def get(self, index):
+    def get(self, index: int) -> Optional[Filter]:
         """
         Return the filter at given index, None if no filter at position.
 
@@ -448,7 +477,7 @@ class ServoDesign:
             )
         return self._filters[index]
 
-    def remove(self, index):
+    def remove(self, index: int) -> None:
         """
         Remove a filter from the servo. Effectively sets the slot at the given index to None.
 
@@ -465,7 +494,7 @@ class ServoDesign:
             )
         self._filters[index] = None
 
-    def is_empty(self):
+    def is_empty(self) -> bool:
         """
         Check whether ServoDesign contains any filter.
         """
@@ -474,7 +503,7 @@ class ServoDesign:
                 return False
         return True
 
-    def __len__(self):
+    def __len__(self) -> int:
         """
         Length of real filters in this ServoDesign.
 
@@ -496,7 +525,12 @@ class ServoDesign:
     # Add Filters the old way
     ####################################
 
-    def integrator(self, corner_frequency, fstop=None, enabled=True):
+    def integrator(
+        self,
+        corner_frequency: float,
+        fstop: Optional[float] = None,
+        enabled: bool = True,
+    ) -> None:
         """
         Add an integrator with corner frequency `corner_frequency`,
         compensated for unity gain at high frequencies.
@@ -511,7 +545,12 @@ class ServoDesign:
         """
         self.add(Integrator(corner_frequency, fstop, enabled))
 
-    def differentiator(self, corner_frequency, fstop=None, enabled=True):
+    def differentiator(
+        self,
+        corner_frequency: float,
+        fstop: Optional[float] = None,
+        enabled: bool = True,
+    ) -> None:
         """
         Add a differentiator with corner frequency `corner_frequency`,
         compensated for unity gain at low frequencies.
@@ -525,7 +564,12 @@ class ServoDesign:
         """
         self.add(Differentiator(corner_frequency, fstop, enabled))
 
-    def lowpass(self, corner_frequency, second_parameter=0.707, enabled=True):
+    def lowpass(
+        self,
+        corner_frequency: float,
+        second_parameter: float = 0.707,
+        enabled: bool = True,
+    ) -> None:
         """
         Add a 2nd-order lowpass filter with variable quality factor `second_parameter`.
 
@@ -538,7 +582,12 @@ class ServoDesign:
         """
         self.add(Lowpass(corner_frequency, second_parameter, enabled))
 
-    def notch(self, corner_frequency, second_parameter=1, enabled=True):
+    def notch(
+        self,
+        corner_frequency: float,
+        second_parameter: float = 1,
+        enabled: bool = True,
+    ) -> None:
         """
         Add a notch filter at frequency `corner_frequency` with a
         quality factor `second_parameter`, where the -3dB filter bandwidth ``bw``
@@ -562,7 +611,7 @@ class ServoDesign:
     # CLASS UTILITY
     ####################################
 
-    def log_gain(self, gain):
+    def log_gain(self, gain: float) -> None:
         """
         Add gain specified in dB (amplitude scale, i.e. 6dB is a factor of 2).
 
@@ -573,7 +622,7 @@ class ServoDesign:
         """
         self.gain *= db.to_lin(gain / 2)
 
-    def zpk(self):
+    def zpk(self) -> Tuple[ndarray, ndarray, float]:
         """
         Return combined zeros, poles and gain for all filters.
 
@@ -622,7 +671,12 @@ class ServoDesign:
 
         self._plant = value
 
-    def _apply(self, freq, ampl=None, phase=None):
+    def _apply(
+        self,
+        freq: Union[ndarray, Float64Index],
+        ampl: Optional[Series] = None,
+        phase: Optional[Series] = None,
+    ) -> DataFrame:
         _, a, p = signal.bode(self.zpk(), freq)  # pylint: disable=unused-variable
 
         df = pd.DataFrame(
@@ -635,7 +689,13 @@ class ServoDesign:
             df["Servo+TF P"] = phase + p
         return df
 
-    def plot(self, freq=None, plot=True, correct_latency=False, **kwargs):
+    def plot(
+        self,
+        freq: Optional[ndarray] = None,
+        plot: bool = True,
+        correct_latency: Union[bool, float] = False,
+        **kwargs,
+    ) -> Union[Figure, DataFrame]:
         """
         Plot the servo response over the frequencies given in `freq`.
 
@@ -690,16 +750,18 @@ class ServoDesign:
         plt = plots.amplitude_phase(df_amplitude, df_phase, **kwargs)
         # add 0dB and -135deg markers
         plt.axes[0].hlines(
-            0, *plt.axes[0].get_xlim(), colors=(0.6, 0.6, 0.6), linestyles="dashed"
+            0, *plt.axes[0].get_xlim(), colors=(0.6, 0.6, 0.6), linestyles="solid"
         )
         plt.axes[1].hlines(
-            -135, *plt.axes[1].get_xlim(), colors=(0.6, 0.6, 0.6), linestyles="dashed"
+            -135, *plt.axes[1].get_xlim(), colors=(0.6, 0.6, 0.6), linestyles="solid"
         )
         return plt
 
     def discrete_form(
-        self, sampling_frequency=SAMPLING_RATE, fs=None,  # pylint: disable=invalid-name
-    ):
+        self,
+        sampling_frequency: float = SAMPLING_RATE,
+        fs: Optional[float] = None,  # pylint: disable=invalid-name
+    ) -> Dict[str, Union[float, List[Dict[str, Union[str, ndarray, bool, float]]]]]:
         """
         Convert the servo and its filters to a digital,
         discrete-time representation in terms of second-order sections
@@ -714,20 +776,20 @@ class ServoDesign:
             warn("fs is deprecated. use sampling_frequency.", DeprecationWarning)
             sampling_frequency = fs
 
-        filters = []
+        filters = [
+            {
+                "description": f.description,
+                "sos": f.discrete_SOS(sampling_frequency).flatten(),
+                "enabled": f.enabled,
+                "index": i,
+            }
+            for i, f in enumerate(self._filters)
+            if f is not None
+        ]
 
-        for i, f in enumerate(self._filters):
-            if f is not None:
-                filters.append(
-                    {
-                        "description": f.description,
-                        "sos": f.discrete_SOS(sampling_frequency).flatten(),
-                        "enabled": f.enabled,
-                        "index": i,
-                    }
-                )
-
-        data = {
+        data: Dict[
+            str, Union[float, List[Dict[str, Union[str, ndarray, bool, float]]]]
+        ] = {
             "sampling_frequency": sampling_frequency,
             "gain": self.gain,
             "filters": filters,
@@ -735,17 +797,25 @@ class ServoDesign:
 
         return data
 
-    def __str__(self):
-        data = []
-        for f in self._filters:
-            if f is not None:
-                data.append([f.description, f.zeros, f.poles, f.gain])
+    def __str__(self) -> str:
+        data = [
+            [f.description, f.zeros, f.poles, f.gain]
+            for f in self._filters
+            if f is not None
+        ]
+
+        description: List[List] = [
+            ["zeros", "poles", "gain"],
+            ["Gain", [], [], self.gain],
+        ]
         return tabulate(
-            [["zeros", "poles", "gain"], ["Gain", [], [], self.gain]] + data,
+            description + data,
             headers="firstrow",
         )
 
-    def __getstate__(self):
+    def __getstate__(
+        self,
+    ) -> Dict[str, Optional[Union[List[Optional[Filter]], float, str]],]:
         state = self.__dict__.copy()
         plant = state["_plant"]
         if isinstance(plant, pd.DataFrame):
@@ -753,7 +823,13 @@ class ServoDesign:
 
         return state
 
-    def __setstate__(self, state):
+    def __setstate__(
+        self,
+        state: Dict[
+            str,
+            Optional[Union[List[Optional[Filter]], float, str]],
+        ],
+    ) -> None:
         if state["_plant"] is not None:
             state["_plant"] = DataContainer.from_json(state["_plant"], orient="split")
         self.__dict__ = state  # pylint: disable=attribute-defined-outside-init
