@@ -1,11 +1,13 @@
+from typing import Tuple
+
 import numpy as np
 
 
-def base_floor(x, base=5):
+def base_floor(x: float, base: int = 5) -> int:
     return base * int(np.floor(x / base))
 
 
-def get_prefix(value, tenth_steps=False):
+def get_prefix(value: float, tenth_steps: bool = False) -> Tuple[float, str]:
     prefixes = {
         24: "Y",
         21: "Z",
@@ -29,6 +31,8 @@ def get_prefix(value, tenth_steps=False):
         -21: "z",
         -24: "y",
     }
+    if value == 0.0:
+        return value, ""
     exponent = base_floor(np.log10(abs(value)), base=3)
     if -3 <= exponent < 3 and tenth_steps:
         exponent = base_floor(np.log10(abs(value)), base=1)
@@ -40,11 +44,10 @@ def get_prefix(value, tenth_steps=False):
     return value, ""
 
 
-def human_readable(value, unit="", tenth_steps=False):
+def human_readable(value: float, unit: str = "", tenth_steps: bool = False) -> str:
     """
     Return an approximate string representation of a value, with
-    appropriate unit prefixes (µ, k, M, ...) added. Values should be between
-    1n and 1PHz. Accuracy is ~0.1%.
+    appropriate unit prefixes (µ, k, M, ...) added. Accuracy is ~0.1%.
 
     Parameters
     ----------
