@@ -1,8 +1,12 @@
-import logging as log
+import logging
 import warnings
 from io import StringIO
 from pathlib import Path
-from typing import BinaryIO, List, TextIO, Union, cast
+from typing import BinaryIO, List, Optional, TextIO, Union, cast
+
+from typeguard import typechecked
+
+from openqlab.io.data_container import DataContainer, DataContainerSeries
 
 from .data_container import DataContainer
 from .importers import utils
@@ -10,12 +14,14 @@ from .importers.ascii_header import ASCII_Header
 
 from .base_importer import BaseImporter, StreamImporter  # isort:skip
 
+log = logging.getLogger(__name__)
+
 
 class UndefinedImporter(Exception):
     pass
 
 
-def list_formats():
+def list_formats() -> None:
     print(
         "The below formats are currently understood by the importer.\n"
         'To import one of these formats, use io.read("filename", importer="<Importer>"),\n'
@@ -31,20 +37,24 @@ def list_formats():
         print("\t", i)
 
 
-def _import(stream: BinaryIO, selected_importers: List[BaseImporter], **kwargs):
+def _import(
+    stream: BinaryIO, selected_importers: List[BaseImporter], **kwargs
+) -> DataContainer:
     for imp in sorted(
         selected_importers, key=lambda importer: importer.PRIORITY, reverse=True
     ):
         try:
             i = imp(stream, **kwargs)  # type: ignore
             return i.read()
-        except (utils.UnknownFileType, UnicodeDecodeError):
+        except (utils.UnknownFileType, UnicodeDecodeError, TypeError):
             pass
+
     raise UndefinedImporter(
         "AutoImporter: unable to find importer for {0}".format(stream)
     )
 
 
+@typechecked
 def read(
     files: Union[str, TextIO, Path, List[Union[str, TextIO, Path]]],
     append: bool = False,
@@ -109,7 +119,7 @@ def read(
     else:
         files_list = [files]
 
-    log.info(files_list)
+    log.debug("file list in io.read: %s", files_list)
 
     importers = BaseImporter.importers()
 
@@ -134,8 +144,9 @@ def read(
     return DataContainer.concat(data, axis=axis)
 
 
+@typechecked
 def reads(data: str, **kwargs) -> Union[DataContainer, List[DataContainer]]:
     stream = StringIO(data)
     stream.name = "StringIO"
-    log.info(f"type of data: {type(stream)}")
+    log.debug("type of data: %s", type(stream))
     return read(stream, **kwargs)
