@@ -3,14 +3,17 @@ import gzip
 import io
 import logging
 from abc import ABC, abstractmethod
+from collections import OrderedDict
 from inspect import isabstract
 from pathlib import Path
 from re import match
 from typing import (
     IO,
+    Any,
     BinaryIO,
     Callable,
     Dict,
+    Iterator,
     List,
     Optional,
     TextIO,
@@ -19,11 +22,16 @@ from typing import (
     cast,
 )
 
-import pyvisa
-
+from openqlab._typing import FilepathOrBuffer
 from openqlab.io.data_container import DataContainer
 from openqlab.io.importers import utils
 
+try:
+    import pyvisa
+except ImportError:
+    pyvisa = None
+
+
 abstract_class_attribute = property(abstractmethod(lambda *args: None))
 
 log = logging.getLogger(__name__)
@@ -39,7 +47,7 @@ class BaseImporter(ABC):
         pass
 
     @classmethod
-    def get_subclasses(cls):
+    def get_subclasses(cls) -> Iterator[Any]:
         for subclass in cls.__subclasses__():
             yield from subclass.get_subclasses()
             yield subclass
@@ -68,14 +76,17 @@ class VisaImporter(BaseImporter, ABC):
 
     IDN_STARTS_WITH = cast(str, abstract_class_attribute)
     IDN_STARTS_WITH.__doc__ = "Start of the device IDN"
-    ADDRESS_STARTS_WITH = cast(str, abstract_class_attribute)
-    ADDRESS_STARTS_WITH.__doc__ = "Mandatory start of the address"
+    ADDRESS_REGEX: str = r"tcpip[0-9]?::"
 
-    def __init__(self, data: Union[str, IO, Path], inst=None):
+    def __init__(self, data: FilepathOrBuffer, inst: None = None):
+        if pyvisa is None:
+            raise utils.UnknownFileType(
+                f"{self.NAME}: pyvisa is not installed. Skipping VISA importers."
+            )
         if not isinstance(data, str):
-            raise utils.UnknownFileType(f"{self.NAME}: not a string")
-        if not data.lower().startswith(self.ADDRESS_STARTS_WITH):
-            raise utils.UnknownFileType(f"{self.NAME}: not a Visa address")
+            raise utils.UnknownFileType(f"{self.NAME}: not a string: {data}")
+        if not match(self.ADDRESS_REGEX, data.lower()):
+            raise utils.UnknownFileType(f"{self.NAME}: not a Visa address: {data}")
 
         rm = pyvisa.ResourceManager("@py")
         if inst is None:
@@ -90,16 +101,18 @@ class VisaImporter(BaseImporter, ABC):
         return self.query("*IDN?").strip()
 
     def query(self, query: str) -> str:
-        return self._inst.query(query)
+        return self._inst.query(query)  # type: ignore
 
-    def write(self, command: str):
-        self._inst.write(command)
+    def write(self, command: str) -> None:
+        self._inst.write(command)  # type: ignore
 
-    def _check_connection(self):
+    def _check_connection(self) -> None:
         try:
             assert self.idn.startswith(self.IDN_STARTS_WITH)
         except (AssertionError, pyvisa.errors.VisaIOError):
-            raise utils.UnknownFileType(f"{self.NAME}: cannot open connection")
+            raise utils.UnknownFileType(
+                f"{self.NAME}: cannot open connection"
+            ) from None
 
 
 class StreamImporter(BaseImporter, ABC):
@@ -112,21 +125,26 @@ class StreamImporter(BaseImporter, ABC):
     STARTING_LINES.__doc__ = "List of strings with a regex to match the first lines."
     HEADER_MAP: Dict[str, Tuple[Callable, Optional[str]]] = NotImplemented
 
-    ENCODING: str = "utf8"
+    ENCODING: str = "utf-8"
     BINARY: bool = False
     HEADER_SPLIT: str = ","
 
-    def __init__(self, data: Union[str, IO, Path]):
-        self._header: Dict = {}
+    def __init__(self, data: FilepathOrBuffer) -> None:
+        self._header: Dict = OrderedDict()
         self._opened_file = False
         self._open_stream(data)
         self._header_lines: List = self._check_header()
+        self._stream: IO
 
-    def __del__(self):
+    def __del__(self) -> None:
         if self._opened_file:
-            self._stream.close()
+            try:
+                self._stream.close()
+            except AttributeError:
+                pass
 
-    def _open_stream(self, data):
+    # @no_type_check
+    def _open_stream(self, data: FilepathOrBuffer) -> None:
         if isinstance(data, Path):
             data = str(data)
         if isinstance(data, str):
@@ -139,20 +157,23 @@ class StreamImporter(BaseImporter, ABC):
                 self._stream = cast(BinaryIO, opener(data, "rb"))
             else:
                 self._stream = cast(
-                    TextIO, opener(data, "rt", encoding=self.ENCODING, newline="")
+                    TextIO,
+                    opener(
+                        data, "rt", encoding=self.ENCODING, newline=""
+                    ),  # newline="" automatically uses the corrent line ending for the OS
                 )
         else:
             self._stream = cast(IO, data)
-        self._stream.seek(0)
+        self._stream.seek(0, 0)
 
     def _check_header(self) -> List:
         lines: List = []  # save the header lines to reuse them if necessary
-        log.info(f"starting lines: {self.STARTING_LINES}")
+        log.debug(f"starting lines: {self.STARTING_LINES}")
         try:
             for i, start in enumerate(self.STARTING_LINES, 1):
                 line = self._stream.readline()
                 lines.append(line)
-                log.info(f"regex: {start}, line: {line}")
+                log.debug("regex: %s, line: %s", start, line)
                 if not isinstance(start, type(line)) or not match(start, line):
                     raise utils.UnknownFileType(
                         f'{self.NAME}: line {i} of file must start with "{start}"'
@@ -171,7 +192,7 @@ class StreamImporter(BaseImporter, ABC):
             pass
         return keyword
 
-    def _read_line(self, line: str):
+    def _read_line(self, line: str) -> None:
         split = line.strip().split(self.HEADER_SPLIT)
         keyword, *values = split
 
@@ -189,7 +210,9 @@ class StreamImporter(BaseImporter, ABC):
             keyword = self._get_key(keyword + "-unit")
             self._insert_header_line(keyword, unit, str)
 
-    def _insert_header_line(self, key: str, value: str, as_type: Union[type, Callable]):
+    def _insert_header_line(
+        self, key: str, value: str, as_type: Union[type, Callable]
+    ) -> None:
         if key in self._header:
             return
 
