@@ -1,9 +1,3 @@
-import glob
-import importlib
-import inspect
-import os
-
-from ..base_importer import BaseImporter
 from . import utils
 from .ascii import ASCII
 from .data_container_csv import DataContainerCSV
@@ -15,7 +9,8 @@ from .keysight_binary import KeysightBinary
 from .keysight_csv import KeysightCSV
 from .keysight_fra import KeysightFRA
 from .keysight_visa import KeysightVisa
-from .rhode_schwarz import RhodeSchwarz
+from .rohde_schwarz import RohdeSchwarz
+from .rohde_schwarz_visa import RohdeSchwarzVisa
 from .tektronix import Tektronix
 from .tektronix_dpx import TektronixDPX
 from .tektronix_spectrogram import TektronixSpectrogram
