@@ -1,6 +1,7 @@
 from typing import List
 
 import pandas as pd
+from pandas.core.frame import DataFrame
 
 from openqlab.io.base_importer import StreamImporter
 from openqlab.io.data_container import DataContainer
@@ -13,14 +14,14 @@ class ASCII(StreamImporter):
     AUTOIMPORTER = False
     STARTING_LINES: List[str] = []
 
-    def _check_header(self):
+    def _check_header(self) -> List:
         for _ in range(11):
             try:
                 line = self._stream.readline()
             except UnicodeDecodeError:
                 raise utils.UnknownFileType(
                     f"'{self.NAME}' importer: cannot decode binary file"
-                )
+                ) from None
 
             col = line.split()
             for item in col:
@@ -29,15 +30,16 @@ class ASCII(StreamImporter):
                 except ValueError:
                     raise utils.UnknownFileType(
                         f"'{self.NAME}' importer: expected plain numeric ASCII"
-                    )
+                    ) from None
         self._stream.seek(0)
+        return []
 
-    def read(self):
+    def read(self) -> DataContainer:
         data = self._read_data()
         output = DataContainer(data)
         return output
 
-    def _read_data(self):
+    def _read_data(self) -> DataFrame:
         xlabel = "x"
         ylabel = utils.get_file_basename(self._stream.name)
         data = pd.read_csv(
