@@ -1,18 +1,17 @@
 import logging
 import re
 from pathlib import Path
-from typing import List
+from typing import List, Match, Optional
 
 import pandas as pd
 
+from openqlab._typing import FilepathOrBuffer
 from openqlab.io.base_importer import StreamImporter
 from openqlab.io.data_container import DataContainer
 from openqlab.io.importers.utils import UnknownFileType
 
 log = logging.getLogger(__name__)
 
-# logger.setLevel(logging.DEBUG)
-
 
 class ASCII_Header(StreamImporter):
     PRIORITY = -5
@@ -22,17 +21,18 @@ class ASCII_Header(StreamImporter):
     HEADER_ESCAPE: str = r"[#$%]"
     LINE_SPLIT: str = r"[,:;\s\t]"
 
-    def __init__(self, stream):
+    def __init__(self, stream: FilepathOrBuffer) -> None:
         super().__init__(stream)
         self._comment = ""
-        self.prefix = None
-        self.header_line = None
+        self.prefix: Optional[str] = None
+        self.header_line: Optional[int] = None
 
-    def read(self):
+    def read(self) -> DataContainer:
         self._read_header()
         if self.header_line is None:
             self.prefix = f"{Path(self._stream.name).stem}_"
         try:
+            log.debug(f"type of header: {type(self._header)}")
             return DataContainer(
                 pd.read_csv(
                     self._stream,
@@ -46,15 +46,18 @@ class ASCII_Header(StreamImporter):
                 header=self._header,
             )
         except Exception as e:
-            raise UnknownFileType(e)
+            raise UnknownFileType from e
 
-    def _read_header(self):
+    def _read_header(self) -> None:
 
-        line = True
+        line = "True"
         while line:
+            line_start_position = self._stream.tell()
             line = self._stream.readline()
             log.debug(rf"line:{repr(line)}")
-            match = re.match(rf"^{self.HEADER_ESCAPE}{{2}}\s*", line)
+            match: Optional[Match[str]] = re.match(
+                rf"^{self.HEADER_ESCAPE}{{2}}\s*", line
+            )
             if match:
                 self._comment += line[match.end() :]
                 continue
@@ -68,12 +71,7 @@ class ASCII_Header(StreamImporter):
             if not re.match(r"[-+]*\d+", line):
                 self.header_line = 0
             break
-        if not self._comment or not self._header:
-            pass
-            # raise UnknownFileType
-        self._header["comment"] = self._comment.strip()
-
-        log.debug(f"position:{self._stream.tell()}")
-        log.debug(f"len(line):{len(line)}")
-
-        self._stream.seek(self._stream.tell() - len(line))
+        if self._comment and self._header:
+            self._header["comment"] = self._comment.strip()
+        log.info("line start cursor position: %s", line_start_position)
+        self._stream.seek(line_start_position)
