@@ -1,4 +1,5 @@
 import pandas as pd
+from pandas.core.frame import DataFrame
 
 from openqlab.io.base_importer import StreamImporter
 from openqlab.io.data_container import DataContainer
@@ -10,14 +11,14 @@ class KeysightCSV(StreamImporter):
     AUTOIMPORTER = True
     STARTING_LINES = ["^x-axis", "^second"]
 
-    def read(self):
+    def read(self) -> DataContainer:
         data = self._read_data()
         output = DataContainer(data, header_type="osci")
         output.header["xUnit"] = "s"
         output.header["yUnit"] = "V"
         return output
 
-    def _read_data(self):
+    def _read_data(self) -> DataFrame:
         xlabel = "Time"
         ylabel = utils.get_file_basename(self._stream.name)
         output = pd.read_csv(
