@@ -11,7 +11,8 @@ class KeysightFRA(StreamImporter):
     ENCODING = "cp1252"
     STARTING_LINES = [r"^\#. Frequency \(Hz\). Amplitude \(Vpp\). Gain \(dB\). Phase"]
 
-    def read(self):
+    def read(self) -> DataContainer:
+        sep = ","
         for s in [",", ";"]:
             if s in self._header_lines[0]:
                 sep = s
