@@ -1,3 +1,4 @@
+from openqlab.io.data_container import DataContainer
 from openqlab.io.importers.tektronix import Tektronix
 
 
@@ -15,7 +16,7 @@ class TektronixDPX(Tektronix):
     }
     xUNIT = "s"
 
-    def read(self):
+    def read(self) -> DataContainer:
         output = super().read()
         output.index.name = "Time"
         return output
