@@ -9,5 +9,5 @@ class ImportFailed(Exception):
     pass
 
 
-def get_file_basename(filename):
+def get_file_basename(filename: str) -> str:
     return os.path.splitext(os.path.basename(filename))[0]
