@@ -1,9 +1,12 @@
 """Plotting scripts in frequency domain."""
-from typing import Optional
+from typing import Optional, Union
 from warnings import warn
 
 import matplotlib.pyplot as plt
 from matplotlib.axes import Axes
+from matplotlib.figure import Figure
+from pandas.core.frame import DataFrame
+from pandas.core.series import Series
 
 from openqlab import analysis
 
@@ -11,7 +14,7 @@ from ..conversion import db
 from ..io import DataContainer
 
 
-def _clamp_phase(phase):
+def _clamp_phase(phase: int) -> float:
     """Return phase with all values mapped to between +/- 180 degrees."""
     warn(
         "DEPRECATED: _clamp_phase is deprecated, \
@@ -22,15 +25,15 @@ def _clamp_phase(phase):
 
 
 def amplitude_phase(
-    amplitude,
-    phase,
-    logf=True,
-    bodeplot=True,
-    clamp_phase=True,
-    dbunits=True,
-    title="Transfer Function",
+    amplitude: Union[DataFrame, Series],
+    phase: Union[DataFrame, Series],
+    logf: bool = True,
+    bodeplot: bool = True,
+    clamp_phase: bool = True,
+    dbunits: bool = True,
+    title: Optional[str] = "Transfer Function",
     **kwargs
-):
+) -> Figure:
     """
     Create an amplitude-phase plot to display transfer function measurements.
 
@@ -67,14 +70,12 @@ def amplitude_phase(
         fig, ax = plt.subplots()
         ax2 = ax.twinx()
 
-    if dbunits:
-        ampl = amplitude
-    else:
-        ampl: DataContainer = db.to_lin(
-            amplitude / 2.0
-        )  # the /2.0 accounts for power/amplitude
+    if not dbunits:
+        amplitude = db.to_lin(amplitude / 2.0)  # the /2.0 accounts for power/amplitude
 
-    ampl.plot(ax=ax, title=title, legend=False, logx=logf, logy=(not dbunits), **kwargs)
+    amplitude.plot(
+        ax=ax, title=title, legend=False, logx=logf, logy=(not dbunits), **kwargs
+    )
     if not bodeplot:
         for _ in range(len(ax.lines) + 1):
             # HACKHACK: advance color cycler so we get continuous colours across
@@ -87,12 +88,18 @@ def amplitude_phase(
     ax.set_ylabel("Amplitude (dB)")
     ax2.set_ylabel(u"Phase (º)")
     ax2.grid(linestyle="--")
-    ax.grid()
+    ax.grid(linestyle="--")
 
     return fig
 
 
-def power_spectrum(data, normalize_to=None, logf=True, title="Power Spectrum"):
+def power_spectrum(
+    data: DataContainer,
+    normalize_to: Optional[Series] = None,
+    logf: bool = True,
+    title: str = "Power Spectrum",
+    ax: Optional[Axes] = None,
+) -> Figure:
     """
     Create a plot for power spectrum data (in dB units), e.g. squeezing measurements over a frequency range.
 
@@ -110,6 +117,8 @@ def power_spectrum(data, normalize_to=None, logf=True, title="Power Spectrum"):
         Use logarithmic frequency axis.
     title: :obj:`str`, optional
         The figure title.
+    ax (Axes, optional):
+        Provide an Axes for advanced figures.
 
     Returns
     -------
@@ -117,7 +126,8 @@ def power_spectrum(data, normalize_to=None, logf=True, title="Power Spectrum"):
         A handle to the created matplotlib figure.
 
     """
-    fig, ax = plt.subplots()
+    if ax is None:
+        _, ax = plt.subplots()
 
     if normalize_to is not None:
         my_data = data.subtract(normalize_to, axis="index")
@@ -130,7 +140,7 @@ def power_spectrum(data, normalize_to=None, logf=True, title="Power Spectrum"):
     ax.set_ylabel(ylabel)
     ax.grid(which="both")
 
-    return fig
+    return ax.figure
 
 
 def relative_input_noise(
@@ -165,8 +175,8 @@ def relative_input_noise(
     """
     data /= volt_dc  # type: ignore
     plot = data.plot(logx=logf, logy=logy, title=title, **kwargs)
-    plot.grid(which="both")
-    plot.set_ylabel(ylabel)
-    plot.set_xlabel("Frequency (Hz)")
+    plot.grid(which="both")  # type: ignore
+    plot.set_ylabel(ylabel)  # type: ignore
+    plot.set_xlabel("Frequency (Hz)")  # type: ignore
 
     return plot
