@@ -1,13 +1,26 @@
 import re
+from typing import Optional, Union
 
+import matplotlib
 import matplotlib.pyplot as plt
 import numpy as np
+from matplotlib.figure import Figure
+from pandas.core.series import Series
 
 from openqlab.conversion import db
 from openqlab.io import DataContainer
 
 
-def zero_span(traces, normalize=True, title="Zero Span"):
+def zero_span(
+    traces: DataContainer,
+    normalize: bool = True,
+    title: Optional[str] = "Zero Span",
+    annotate: bool = True,
+    digits_number: int = 2,
+    x_pos: int = 20,
+    alignment: str = "left",
+    ax: matplotlib.axes.Axes = None,
+) -> Figure:
     """
     Create a sensible plot for zero-span squeezing `scope` measurements.
 
@@ -17,13 +30,25 @@ def zero_span(traces, normalize=True, title="Zero Span"):
         normalize (bool, optional):
             Normalize the data to the vacuum trace, which requires a column
             name starting with "*vac*".
-        title (str, optional): The figure title.
+        title (str, optional):
+            The figure title.
+        annotate (bool, optional):
+            Add noise values to the plot.
+        digits_number (int, optional):
+            Number of digits for the calculated dB values.
+        x_pos (int, optional):
+            Distance between graph and labels.
+        alignment (str, optional):
+            Horizontal alignment of labels.
+            Can be "left", "center" or "right".
+        ax (Axes, optional):
+            Provide an Axes for advanced figures.
 
     Returns:
-        matplotlib.figure.Figure: A handle to the created matplotlib figure.
+        matplotlib.figure.Figure: A handle to the created matplotlib Figure.
     """
-    fig, ax = plt.subplots()
-    plt.subplots_adjust(right=0.85)
+    if ax is None:
+        _, ax = plt.subplots()
 
     ylabel = "Noise Power (dBm)"
     unit = "dBm"
@@ -49,25 +74,27 @@ def zero_span(traces, normalize=True, title="Zero Span"):
     ii = 0
     for trace in traces:
         std = traces[trace].std()
-        if std < 1:
+        if annotate and std < 1:
             mean_val = db.mean(traces[trace])
             ax.annotate(
-                "{0:.2f}{1}".format(mean_val, unit),
+                f"{mean_val:.{digits_number}f} {unit}",
                 (traces.index[-1], mean_val),
-                (20, 0),
+                (x_pos, 0),
                 textcoords="offset points",
-                horizontalalignment="left",
+                horizontalalignment=alignment,
                 verticalalignment="center",
                 color=ax.lines[ii].get_color(),
             )
         ii += 1
-    return fig
+    return ax.figure
 
 
 # SCOPE
 #
 # A very simple plot of time-domain traces, e.g. from an oscilloscope
-def scope(traces, title="Oscilloscope View"):
+def scope(
+    traces: Union[Series, DataContainer], title: str = "Oscilloscope View"
+) -> Figure:
     """
     Create plot for oscilloscope data.
 
